@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.c,v 1.78 2004/10/23 19:40:16 olivier Exp $
+/*      $Id: xfwm4_plugin.c,v 1.80 2004/11/27 20:40:43 pollux Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -965,9 +965,10 @@ keybinding_selection_changed (GtkTreeSelection * selection, gpointer data)
             
             if (ti)
             {
+	        loadtheme_in_treeview (ti, itf);
                 gtk_widget_set_sensitive (itf->treeview3, ti->user_writable);
                 gtk_widget_set_sensitive (itf->treeview4, ti->user_writable);
-                loadtheme_in_treeview (ti, itf);
+		gtk_widget_set_sensitive (itf->del_button, ti->user_writable);
             }
         }
     }
@@ -1369,6 +1370,8 @@ create_dialog (McsPlugin * mcs_plugin)
     GtkWidget *vbox6;
     GtkWidget *vbox7;
     GtkWidget *vbox8;
+    GtkWidget *vbox9;
+    GtkTreeViewColumn *hidden_column;
 
     GtkCellRenderer *renderer;
     GtkListStore *model;
@@ -1457,25 +1460,37 @@ create_dialog (McsPlugin * mcs_plugin)
     gtk_widget_show (hbox);
     gtk_container_add (GTK_CONTAINER (notebook), hbox);
 
+    vbox8 = gtk_vbox_new (FALSE, BORDER);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox8), BORDER);
+    gtk_box_pack_start (GTK_BOX (hbox), vbox8, FALSE, FALSE, 0);
+    gtk_widget_show (vbox8);
+
     dialog->scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
     gtk_container_set_border_width (GTK_CONTAINER (dialog->scrolledwindow2), BORDER);
     gtk_widget_show (dialog->scrolledwindow2);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (dialog->scrolledwindow2), GTK_SHADOW_IN);
-    gtk_box_pack_start (GTK_BOX (hbox), dialog->scrolledwindow2, FALSE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox8), dialog->scrolledwindow2, TRUE, TRUE, 0);
 
     dialog->treeview2 = gtk_tree_view_new ();
     gtk_widget_show (dialog->treeview2);
     gtk_container_add (GTK_CONTAINER (dialog->scrolledwindow2), dialog->treeview2);
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dialog->treeview2), FALSE);
 
-    vbox8 = gtk_vbox_new (FALSE, BORDER);
-    gtk_container_set_border_width (GTK_CONTAINER (vbox8), BORDER);
-    gtk_box_pack_start (GTK_BOX (hbox), vbox8, TRUE, TRUE, 0);
-    gtk_widget_show (vbox8);
+    dialog->add_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
+    gtk_widget_show (dialog->add_button);
+    gtk_box_pack_start (GTK_BOX (vbox8), dialog->add_button, FALSE, FALSE, 0);
+    dialog->del_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+    gtk_widget_show (dialog->del_button);
+    gtk_box_pack_start (GTK_BOX (vbox8), dialog->del_button, FALSE, FALSE, 0);
+
+    vbox9 = gtk_vbox_new (FALSE, BORDER);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox9), BORDER);
+    gtk_box_pack_start (GTK_BOX (hbox), vbox9, TRUE, TRUE, 0);
+    gtk_widget_show (vbox9);
 
     frame = xfce_framebox_new (_("Window shortcuts"), FALSE);
     gtk_widget_show (frame);
-    gtk_box_pack_start (GTK_BOX (vbox8), frame, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox9), frame, TRUE, TRUE, 0);
 
     dialog->scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dialog->scrolledwindow3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -1484,7 +1499,7 @@ create_dialog (McsPlugin * mcs_plugin)
     xfce_framebox_add (XFCE_FRAMEBOX (frame), dialog->scrolledwindow3);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (dialog->scrolledwindow3), GTK_SHADOW_IN);
 
-    model = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
+    model = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     dialog->treeview3 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
     gtk_widget_show (dialog->treeview3);
     gtk_container_add (GTK_CONTAINER (dialog->scrolledwindow3), dialog->treeview3);
@@ -1499,11 +1514,16 @@ create_dialog (McsPlugin * mcs_plugin)
     g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_SHORTCUT);
 
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->treeview3), -1, _("Shortcut"), renderer, "text", COLUMN_SHORTCUT, NULL);
+    /* command name hidden column */
+    hidden_column = gtk_tree_view_column_new_with_attributes ("name", renderer, "text", COLUMN_NAME, NULL);
+    gtk_tree_view_column_set_visible (hidden_column, FALSE);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->treeview3), hidden_column);
+
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dialog->treeview3), TRUE);
 
     frame = xfce_framebox_new (_("Command shortcuts"), FALSE);
     gtk_widget_show (frame);
-    gtk_box_pack_start (GTK_BOX (vbox8), frame, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox9), frame, TRUE, TRUE, 0);
 
     dialog->scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dialog->scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -1512,7 +1532,7 @@ create_dialog (McsPlugin * mcs_plugin)
     xfce_framebox_add (XFCE_FRAMEBOX (frame), dialog->scrolledwindow4);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (dialog->scrolledwindow4), GTK_SHADOW_IN);
 
-    model = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
+    model = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     dialog->treeview4 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
     gtk_widget_show (dialog->treeview4);
     gtk_container_add (GTK_CONTAINER (dialog->scrolledwindow4), dialog->treeview4);
@@ -1525,8 +1545,12 @@ create_dialog (McsPlugin * mcs_plugin)
     /* shortcut column */
     renderer = gtk_cell_renderer_text_new ();
     g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_SHORTCUT);
-
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->treeview4), -1, _("Shortcut"), renderer, "text", COLUMN_SHORTCUT, NULL);
+    /* command name hidden column */
+    hidden_column = gtk_tree_view_column_new_with_attributes ("name", renderer, "text", COLUMN_NAME, NULL);
+    gtk_tree_view_column_set_visible (hidden_column, FALSE);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->treeview4), hidden_column);
+    
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dialog->treeview4), TRUE);
     /* popup menu */
     dialog->popup_menu = gtk_menu_new ();
@@ -1825,6 +1849,8 @@ setup_dialog (Itf * itf)
     g_signal_connect (G_OBJECT (itf->treeview2), "button-press-event", G_CALLBACK (cb_popup_menu), itf);
     g_signal_connect (G_OBJECT (itf->popup_add_menuitem), "activate", G_CALLBACK (cb_popup_add_menu), itf);
     g_signal_connect (G_OBJECT (itf->popup_del_menuitem), "activate", G_CALLBACK (cb_popup_del_menu), itf);
+    g_signal_connect (G_OBJECT (itf->add_button), "clicked", G_CALLBACK (cb_popup_add_menu), itf);
+    g_signal_connect (G_OBJECT (itf->del_button), "clicked", G_CALLBACK (cb_popup_del_menu), itf);
 
     g_signal_connect (G_OBJECT (itf->treeview3), "row-activated", G_CALLBACK (cb_activate_treeview3), itf);
     g_signal_connect (G_OBJECT (itf->treeview4), "row-activated", G_CALLBACK (cb_activate_treeview4), itf);
