@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.h,v 1.13 2004/11/11 14:42:34 pollux Exp $
+/*      $Id: xfwm4_plugin.h,v 1.15 2004/11/27 20:40:43 pollux Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -105,6 +105,8 @@ struct _Itf
     GtkWidget *popup_menu;
     GtkWidget* popup_add_menuitem;
     GtkWidget* popup_del_menuitem;
+    GtkWidget* add_button;
+    GtkWidget* del_button;
 
 };
 
@@ -118,6 +120,7 @@ enum
 {
     COLUMN_COMMAND,
     COLUMN_SHORTCUT,
+    COLUMN_NAME,
     NUM_COLUMNS
 };
 
