@@ -1,4 +1,4 @@
-/*      $Id: display.c,v 1.11 2004/10/24 13:32:37 olivier Exp $
+/*      $Id: display.c,v 1.18 2004/12/09 22:44:20 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #endif
 
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 #include <gtk/gtk.h>
@@ -49,6 +50,108 @@ handleXError (Display * dpy, XErrorEvent * err)
     return 0;
 }
 
+static gboolean
+myDisplayInitAtoms (DisplayInfo *display_info)
+{
+    char *atom_names[] = {
+        "GNOME_PANEL_DESKTOP_AREA",
+        "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
+        "KWM_WIN_ICON",
+        "_MOTIF_WM_HINTS",
+        "_NET_ACTIVE_WINDOW",
+        "_NET_CLIENT_LIST",
+        "_NET_CLIENT_LIST_STACKING",
+        "_NET_CLOSE_WINDOW",
+        "_NET_CURRENT_DESKTOP",
+        "_NET_DESKTOP_GEOMETRY",
+        "_NET_DESKTOP_LAYOUT",
+        "_NET_DESKTOP_NAMES",
+        "_NET_DESKTOP_VIEWPORT",
+        "_NET_FRAME_EXTENTS",
+        "_NET_NUMBER_OF_DESKTOPS",
+        "_NET_REQUEST_FRAME_EXTENTS",
+        "_NET_SHOWING_DESKTOP",
+        "_NET_STARTUP_ID",
+        "_NET_SUPPORTED",
+        "_NET_SUPPORTING_WM_CHECK",
+        "_NET_SYSTEM_TRAY_MANAGER",
+        "_NET_SYSTEM_TRAY_OPCODE",
+        "_NET_WM_ACTION_CHANGE_DESKTOP",
+        "_NET_WM_ACTION_CLOSE",
+        "_NET_WM_ACTION_MAXIMIZE_HORZ",
+        "_NET_WM_ACTION_MAXIMIZE_VERT",
+        "_NET_WM_ACTION_MOVE",
+        "_NET_WM_ACTION_RESIZE",
+        "_NET_WM_ACTION_SHADE",
+        "_NET_WM_ACTION_STICK",
+        "_NET_WM_ALLOWED_ACTIONS",
+        "_NET_WM_CONTEXT_HELP",
+        "_NET_WM_DESKTOP",
+        "_NET_WM_ICON",
+        "_NET_WM_ICON_GEOMETRY",
+        "_NET_WM_ICON_NAME",
+        "_NET_WM_MOVERESIZE",
+        "_NET_WM_NAME",
+        "_NET_WM_WINDOW_OPACITY",
+        "_NET_WM_STATE",
+        "_NET_WM_STATE_ABOVE",
+        "_NET_WM_STATE_BELOW",
+        "_NET_WM_STATE_DEMANDS_ATTENTION",
+        "_NET_WM_STATE_FULLSCREEN",
+        "_NET_WM_STATE_HIDDEN",
+        "_NET_WM_STATE_MAXIMIZED_HORZ",
+        "_NET_WM_STATE_MAXIMIZED_VERT",
+        "_NET_WM_STATE_MODAL",
+        "_NET_WM_STATE_SHADED",
+        "_NET_WM_STATE_SKIP_PAGER",
+        "_NET_WM_STATE_SKIP_TASKBAR",
+        "_NET_WM_STATE_STICKY",
+        "_NET_WM_STRUT",
+        "_NET_WM_STRUT_PARTIAL",
+        "_NET_WM_USER_TIME",
+        "_NET_WM_WINDOW_TYPE",
+        "_NET_WM_WINDOW_TYPE_DESKTOP",
+        "_NET_WM_WINDOW_TYPE_DIALOG",
+        "_NET_WM_WINDOW_TYPE_DOCK",
+        "_NET_WM_WINDOW_TYPE_MENU",
+        "_NET_WM_WINDOW_TYPE_NORMAL",
+        "_NET_WM_WINDOW_TYPE_SPLASH",
+        "_NET_WM_WINDOW_TYPE_TOOLBAR",
+        "_NET_WM_WINDOW_TYPE_UTILITY",
+        "_NET_WORKAREA",
+        "PIXMAP",
+        "SM_CLIENT_ID",
+        "UTF8_STRING",
+        "_WIN_CLIENT_LIST",
+        "_WIN_DESKTOP_BUTTON_PROXY",
+        "_WIN_HINTS",
+        "_WIN_LAYER",
+        "_WIN_PROTOCOLS",
+        "_WIN_STATE",
+        "_WIN_SUPPORTING_WM_CHECK",
+        "_WIN_WORKSPACE",
+        "_WIN_WORKSPACE_COUNT",
+        "WM_CHANGE_STATE",
+        "WM_CLIENT_LEADER",
+        "WM_COLORMAP_WINDOWS",
+        "WM_DELETE_WINDOW",
+        "WM_HINTS",
+        "WM_PROTOCOLS",
+        "WM_STATE",
+        "WM_TAKE_FOCUS",
+        "WM_TRANSIENT_FOR",
+        "WM_WINDOW_ROLE",
+        "_XROOTPMAP_ID",
+        "_XSETROOT_ID"
+    };
+    
+    g_assert (NB_ATOMS == G_N_ELEMENTS (atom_names));
+    return (XInternAtoms (display_info->dpy, 
+                          atom_names, 
+                          NB_ATOMS,
+                          FALSE, display_info->atoms) != 0);
+}
+
 DisplayInfo *
 myDisplayInit (GdkDisplay *gdisplay)
 {
@@ -62,11 +165,18 @@ myDisplayInit (GdkDisplay *gdisplay)
 
     XSetErrorHandler (handleXError);
 
+    /* Initialize internal atoms */
+    if (!myDisplayInitAtoms (display))
+    {
+        g_warning ("Some internal atoms were not properly created.");
+    }
+
+    /* Test XShape extension support */
     display->shape = 
         XShapeQueryExtension (display->dpy, &display->shape_event, &dummy);
     if (!display->shape)
     {
-        g_warning ("%s: The display does not support the XShape extension.", PACKAGE);
+        g_warning ("The display does not support the XShape extension.");
     }
 
     display->root_cursor = 
@@ -96,13 +206,7 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->xgrabcount = 0;
     display->dbl_click_time = 300;
     display->nb_screens = 0;
-
-    initICCCMHints (display->dpy);
-    initMotifHints (display->dpy);
-    initGnomeHints (display->dpy);
-    initNetHints   (display->dpy);
-    initKDEHints   (display->dpy);
-    initMiscHints  (display->dpy);
+    display->current_time = CurrentTime;
     
     compositorInitDisplay (display);
 
@@ -387,3 +491,51 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 
     return NULL;
 }
+
+Time 
+myDisplayUpdateCurentTime (DisplayInfo *display, XEvent *ev)
+{
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
+    
+    switch (ev->type)
+    {
+        case KeyPress:
+        case KeyRelease:
+            display->current_time = (Time) ev->xkey.time;
+            break;
+        case ButtonPress:
+        case ButtonRelease:
+            display->current_time = (Time) ev->xbutton.time;
+            break;
+        case MotionNotify:
+            display->current_time = (Time) ev->xmotion.time;
+            break;
+        case EnterNotify:
+        case LeaveNotify:
+            display->current_time = (Time) ev->xcrossing.time;
+            break;
+        case PropertyNotify:
+            display->current_time = (Time) ev->xproperty.time;
+            break;
+        case SelectionClear:
+            display->current_time = (Time) ev->xselectionclear.time;
+            break;
+        case SelectionRequest:
+            display->current_time = (Time) ev->xselectionrequest.time;
+            break;
+        case SelectionNotify:
+            display->current_time = (Time) ev->xselection.time;
+            break;
+        default:
+            break;
+    }
+    return display->current_time;
+}
+
+Time 
+myDisplayGetCurrentTime (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
+
+    return (Time) display->current_time;
+}
