@@ -1,4 +1,4 @@
-/*      $Id: events.c,v 1.282 2004/11/04 21:51:38 olivier Exp $
+/*      $Id: events.c,v 1.293 2004/12/11 16:41:06 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ XfwmButtonClickType;
 typedef struct _XfwmButtonClickData XfwmButtonClickData;
 struct _XfwmButtonClickData
 {
-    Time event_time;
+    DisplayInfo *display_info;
     Window w;
     guint button;
     gboolean allow_double_click;
@@ -135,13 +135,15 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
     XfceFilterStatus status = XEV_FILTER_STOP;
     XfwmButtonClickData *passdata = (XfwmButtonClickData *) data;
     
+    /* Update the display time */
+    myDisplayUpdateCurentTime (passdata->display_info, xevent);
+
     if ((xevent->type == ButtonRelease) || (xevent->type == ButtonPress))
     {
         if (xevent->xbutton.button == passdata->button)
         {
             passdata->clicks++;
         }
-        passdata->event_time = xevent->xbutton.time;
         if (((XfwmButtonClickType) passdata->clicks == XFWM_BUTTON_DOUBLE_CLICK)
             || (!(passdata->allow_double_click) && 
                  (XfwmButtonClickType) passdata->clicks == XFWM_BUTTON_CLICK))
@@ -184,7 +186,7 @@ static XfwmButtonClickType
 typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_double_click)
 {
     int g = GrabSuccess;
-    DisplayInfo *display_info;
+    DisplayInfo *display_info = NULL;
     XfwmButtonClickData passdata;
 
     g_return_val_if_fail (screen_info != NULL, XFWM_BUTTON_UNDEFINED);
@@ -203,6 +205,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
         return XFWM_BUTTON_UNDEFINED;
     }
 
+    passdata.display_info = display_info;
     passdata.button = ev->xbutton.button;
     passdata.w = w;
     passdata.x = ev->xbutton.x_root;
@@ -211,7 +214,6 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     passdata.ycurrent = passdata.y;
     passdata.clicks = 1;
     passdata.allow_double_click = allow_double_click;
-    passdata.event_time = ev->xbutton.time;
     passdata.timeout = g_timeout_add_full (0, display_info->dbl_click_time, 
                                               (GtkFunction) typeOfClick_break, 
                                               (gpointer) &passdata, NULL);
@@ -222,7 +224,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     xfce_pop_event_filter (display_info->xfilter);
     TRACE ("leaving typeOfClick loop");
 
-    XUngrabPointer (display_info->dpy, passdata.event_time);
+    XUngrabPointer (display_info->dpy, myDisplayGetCurrentTime (display_info));
     XFlush (display_info->dpy);
     return (XfwmButtonClickType) passdata.clicks;
 }
@@ -263,7 +265,6 @@ raise_cb (gpointer data)
     if (c)
     {
         clientRaise (c);
-        clientPassGrabMouseButton (c);
     }
     return (TRUE);
 }
@@ -387,7 +388,10 @@ handleMotionNotify (DisplayInfo *display_info, XMotionEvent * ev)
                 }
             }
             while (XCheckWindowEvent(display_info->dpy, ev->window, PointerMotionMask, (XEvent *) ev))
-                ; /* Skip event */
+            {
+                /* Update the display time */
+                myDisplayUpdateCurentTime (display_info, (XEvent *) ev);
+            }
         }
         if (edge_scroll_y > screen_info->params->wrap_resistance)
         {
@@ -407,7 +411,10 @@ handleMotionNotify (DisplayInfo *display_info, XMotionEvent * ev)
                 }
             }
             while (XCheckWindowEvent(display_info->dpy, ev->window, PointerMotionMask, (XEvent *) ev))
-                ; /* Skip event */
+            {
+                /* Update the display time */
+                myDisplayUpdateCurentTime (display_info, (XEvent *) ev);
+            }
         }
     }
 }
@@ -495,11 +502,9 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 break;
             case KEY_RAISE_WINDOW:
                 clientRaise (c);
-                clientPassGrabMouseButton (NULL);
                 break;
             case KEY_LOWER_WINDOW:
                 clientLower (c);
-                clientPassGrabMouseButton (NULL);
                 break;
             case KEY_TOGGLE_FULLSCREEN:
                 clientToggleFullscreen (c);
@@ -634,7 +639,6 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
         if (tclick == XFWM_BUTTON_CLICK)
         {
             clientLower (c);
-            clientPassGrabMouseButton (NULL);
         }
         else if (tclick != XFWM_BUTTON_UNDEFINED)
         {
@@ -647,10 +651,9 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
         {
             if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
             {
-                clientSetFocus (c->screen_info, c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+                clientSetFocus (c->screen_info, c, ev->time, NO_FOCUS_FLAG);
             }
             clientRaise (c);
-            clientPassGrabMouseButton (c);
         }
         if ((ev->button == Button1) || (ev->button == Button3))
         {
@@ -678,7 +681,6 @@ button1Action (Client * c, XButtonEvent * ev)
         clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
     }
     clientRaise (c);
-    clientPassGrabMouseButton (c);
 
     memcpy(&copy_event, ev, sizeof(XEvent));
     tclick = typeOfClick (screen_info, c->window, &copy_event, TRUE);
@@ -728,7 +730,6 @@ titleButton (Client * c, int state, XButtonEvent * ev)
     else if (ev->button == Button2)
     {
         clientLower (c);
-        clientPassGrabMouseButton (NULL);
     }
     else if (ev->button == Button3)
     {
@@ -756,7 +757,6 @@ titleButton (Client * c, int state, XButtonEvent * ev)
             if (screen_info->params->raise_on_click)
             {
                 clientRaise (c);
-                clientPassGrabMouseButton (c);
             }
             ev->window = ev->root;
             if (screen_info->button_handler_id)
@@ -834,13 +834,10 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         return;
     }
 
-    /* Clear timeout */
-    clear_timeout ();
-
     c = myDisplayGetClientFromWindow (display_info, ev->window, ANY);
     if (c)
     {
-        state = ev->state &  MODIFIER_MASK;
+        state = ev->state & MODIFIER_MASK;
         win = ev->subwindow;
         screen_info = c->screen_info;
 
@@ -851,7 +848,6 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         else if ((ev->button == Button2) && (state == AltMask) && (screen_info->params->easy_click))
         {
             clientLower (c);
-            clientPassGrabMouseButton (NULL);
         }
         else if ((ev->button == Button3) && (state == AltMask) && (screen_info->params->easy_click))
         {
@@ -882,8 +878,9 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
                 }
                 if (screen_info->params->raise_on_click)
                 {
+                    /* Clear timeout */
+                    clear_timeout ();
                     clientRaise (c);
-                    clientPassGrabMouseButton (c);
                 }
                 clientButtonPress (c, win, ev);
             }
@@ -919,8 +916,9 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
                     }
                     if (screen_info->params->raise_on_click)
                     {
+                        /* Clear timeout */
+                        clear_timeout ();
                         clientRaise (c);
-                        clientPassGrabMouseButton (c);
                     }
                     ev->window = ev->root;
                     if (screen_info->button_handler_id)
@@ -970,10 +968,9 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         }
         else if (ev->window == c->window)
         {
-            if ((screen_info->params->raise_with_any_button) || 
-                (ev->button == Button1))
+            clientPassGrabMouseButton (c);
+            if ((screen_info->params->raise_with_any_button) || (ev->button == Button1))
             {
-                clientPassGrabMouseButton (c);
                 if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
                 {
                     clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
@@ -981,6 +978,8 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
                 if ((screen_info->params->raise_on_click) || 
                     !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
                 {
+                    /* Clear timeout */
+                    clear_timeout ();
                     clientRaise (c);
                 }
             }
@@ -1016,7 +1015,7 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
     }
     else
     {
-        XUngrabPointer (display_info->dpy, GDK_CURRENT_TIME);
+        XUngrabPointer (display_info->dpy, CurrentTime);
         XSendEvent (display_info->dpy, screen_info->gnome_win, FALSE, SubstructureNotifyMask, (XEvent *) ev);
     }
 }
@@ -1252,6 +1251,9 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
     /* Compress events - logic taken from kwin */
     while (XCheckTypedWindowEvent (display_info->dpy, ev->window, ConfigureRequest, &otherEvent))
     {
+        /* Update the display time */
+        myDisplayUpdateCurentTime (display_info, &otherEvent);
+
         if (otherEvent.xconfigurerequest.value_mask == ev->value_mask)
         {
             ev = &otherEvent.xconfigurerequest;
@@ -1421,10 +1423,6 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
             if (!(c->type & (WINDOW_DOCK | WINDOW_DESKTOP)))
             {
                 clientSetFocus (c->screen_info, c, ev->time, FOCUS_FORCE);
-                if (!(screen_info->params->raise_on_click))
-                {
-                    clientPassGrabMouseButton (c);
-                }
             }
         }
     }
@@ -1507,7 +1505,7 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
         c = clientGetFocus ();
         if (c)
         {
-            clientSetFocus (c->screen_info, c, GDK_CURRENT_TIME, FOCUS_FORCE);
+            clientSetFocus (c->screen_info, c, myDisplayGetCurrentTime (display_info), FOCUS_FORCE);
         }
         return;
     }
@@ -1532,7 +1530,6 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
             (last_raised != NULL) && (c != last_raised))
         {
             clientRaise (c);
-            clientPassGrabMouseButton (c);
         }
         if (screen_info->params->raise_on_focus)
         {
@@ -1583,7 +1580,6 @@ handleFocusOut (DisplayInfo *display_info, XFocusChangeEvent * ev)
         {
             TRACE ("focus lost from \"%s\" (0x%lx)", c->name, c->window);
             clientUpdateFocus (c->screen_info, NULL, NO_FOCUS_FLAG);
-            clientPassGrabMouseButton (NULL);
             /* Clear timeout */
             clear_timeout ();
         }
@@ -1609,18 +1605,18 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
             TRACE ("client \"%s\" (0x%lx) has received a XA_WM_NORMAL_HINTS notify", c->name, c->window);
             clientGetWMNormalHints (c, TRUE);
         }
-        else if ((ev->atom == XA_WM_NAME) || (ev->atom == net_wm_name))
+        else if ((ev->atom == XA_WM_NAME) || (ev->atom == display_info->atoms[NET_WM_NAME]))
         {
             TRACE ("client \"%s\" (0x%lx) has received a XA_WM_NAME notify", c->name, c->window);
             if (c->name)
             {
                 free (c->name);
             }
-            getWindowName (display_info->dpy, c->window, &c->name);
+            getWindowName (display_info, c->window, &c->name);
             FLAG_SET (c->flags, CLIENT_FLAG_NAME_CHANGED);
             frameDraw (c, TRUE, FALSE);
         }
-        else if (ev->atom == motif_wm_hints)
+        else if (ev->atom == display_info->atoms[MOTIF_WM_HINTS])
         {
             TRACE ("client \"%s\" (0x%lx) has received a motif_wm_hints notify", c->name, c->window);
             clientGetMWMHints (c, TRUE);
@@ -1638,38 +1634,39 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
             }
             clientUpdateUrgency (c);
         }
-        else if (ev->atom == wm_protocols)
+        else if (ev->atom == display_info->atoms[WM_PROTOCOLS])
         {
             TRACE ("client \"%s\" (0x%lx) has received a wm_protocols notify", c->name, c->window);
             clientGetWMProtocols (c);
         }
-        else if (ev->atom == wm_transient_for)
+        else if (ev->atom == display_info->atoms[WM_TRANSIENT_FOR])
         {
             TRACE ("client \"%s\" (0x%lx) has received a wm_transient_for notify", c->name, c->window);
-            getTransientFor (display_info->dpy, c->screen_info->screen, c->window, &w);
+            getTransientFor (display_info, c->screen_info->xroot, c->window, &w);
             if (clientCheckTransientWindow (c, w))
             {
                 c->transient_for = w;
                 clientRaise (c);
             }
         }
-        else if (ev->atom == win_hints)
+        else if (ev->atom == display_info->atoms[WIN_HINTS])
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_hints notify", c->name, c->window);
-            getHint (display_info->dpy, c->window, win_hints, &c->win_hints);
+            getHint (display_info, c->window, WIN_HINTS, &c->win_hints);
         }
-        else if (ev->atom == net_wm_window_type)
+        else if (ev->atom == display_info->atoms[NET_WM_WINDOW_TYPE])
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_window_type notify", c->name, c->window);
             clientGetNetWmType (c);
             frameDraw (c, TRUE, FALSE);
         }
-        else if ((ev->atom == net_wm_strut) || (ev->atom == net_wm_strut_partial))
+        else if ((ev->atom == display_info->atoms[NET_WM_STRUT]) || 
+                 (ev->atom == display_info->atoms[NET_WM_STRUT_PARTIAL]))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_strut notify", c->name, c->window);
             clientGetNetStruts (c);
         }
-        else if (ev->atom == wm_colormap_windows)
+        else if (ev->atom == display_info->atoms[WM_COLORMAP_WINDOWS])
         {
             TRACE ("client \"%s\" (0x%lx) has received a wm_colormap_windows notify", c->name, c->window);
             clientUpdateColormaps (c);
@@ -1678,32 +1675,32 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
                 clientInstallColormaps (c);
             }
         }
-        else if (ev->atom == net_wm_user_time)
+        else if (ev->atom == display_info->atoms[NET_WM_USER_TIME])
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_user_time notify", c->name, c->window);
-            if (getNetWMUserTime (display_info->dpy, c->window, &c->user_time))
+            if (getNetWMUserTime (display_info, c->window, &c->user_time))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_HAS_USER_TIME);
             }
         }
-        else if (ev->atom == net_wm_opacity)
+        else if (ev->atom == display_info->atoms[NET_WM_WINDOW_OPACITY])
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_opacity notify", c->name, c->window);
-            if (!getOpacity (display_info->dpy, c->window, &c->opacity))
+            if (!getOpacity (display_info, c->window, &c->opacity))
             {
                 c->opacity =  NET_WM_OPAQUE;
             }
             compositorWindowSetOpacity (display_info, c->frame, c->opacity);
         }
 #ifdef HAVE_STARTUP_NOTIFICATION
-        else if (ev->atom == net_startup_id)
+        else if (ev->atom == display_info->atoms[NET_STARTUP_ID])
         {
             if (c->startup_id)
             {
                 free (c->startup_id);
                 c->startup_id = NULL;
             }
-            getWindowStartupId (display_info->dpy, c->window, &c->startup_id);
+            getWindowStartupId (display_info, c->window, &c->startup_id);
         }
 #endif
         return;
@@ -1715,24 +1712,24 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         return;
     }
 
-    if (ev->atom == net_desktop_names)
+    if (ev->atom == display_info->atoms[NET_DESKTOP_NAMES])
     {
         TRACE ("root has received a net_desktop_names notify");
-        if (getUTF8String (display_info->dpy, screen_info->xroot, net_desktop_names, &names, &length))
+        if (getUTF8String (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &length))
         {
             workspaceSetNames (screen_info, names, length);
         }
     }
-    else if (ev->atom == gnome_panel_desktop_area)
+    else if (ev->atom == display_info->atoms[GNOME_PANEL_DESKTOP_AREA])
     {
         TRACE ("root has received a gnome_panel_desktop_area notify");
-        getGnomeDesktopMargins (display_info->dpy, screen_info->screen, screen_info->gnome_margins);
+        getGnomeDesktopMargins (display_info, screen_info->xroot, screen_info->gnome_margins);
         workspaceUpdateArea (screen_info);
     }
-    else if (ev->atom == net_desktop_layout)
+    else if (ev->atom == display_info->atoms[NET_DESKTOP_LAYOUT])
     {
         TRACE ("root has received a net_desktop_layout notify");
-        getDesktopLayout(display_info->dpy, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
+        getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
     }
 }
 
@@ -1751,21 +1748,20 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         screen_info = c->screen_info;
         is_transient = clientIsValidTransientOrModal (c);
         
-        if ((ev->message_type == wm_change_state) && (ev->format == 32) && (ev->data.l[0] == IconicState))
+        if ((ev->message_type == display_info->atoms[WM_CHANGE_STATE]) && (ev->format == 32) && (ev->data.l[0] == IconicState))
         {
             TRACE ("client \"%s\" (0x%lx) has received a wm_change_state event", c->name, c->window);
-            if (!FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED) && 
-                 CLIENT_CAN_HIDE_WINDOW (c))
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED) &&  CLIENT_CAN_HIDE_WINDOW (c))
             {
                 clientHide (c, c->win_workspace, TRUE);
             }
         }
-        else if ((ev->message_type == win_state) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[WIN_STATE]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_state event", c->name, c->window);
             clientUpdateWinState (c, ev);
         }
-        else if ((ev->message_type == win_layer) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[WIN_LAYER]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_layer event", c->name, c->window);
             if ((ev->data.l[0] != c->win_layer) && !is_transient)
@@ -1773,7 +1769,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 clientSetLayer (c, ev->data.l[0]);
             }
         }
-        else if ((ev->message_type == win_workspace) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[WIN_WORKSPACE]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_workspace event", c->name, c->window);
             if ((ev->data.l[0] != c->win_workspace) && !is_transient)
@@ -1781,15 +1777,14 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 clientSetWorkspace (c, ev->data.l[0], TRUE);
             }
         }
-        else if ((ev->message_type == net_wm_desktop) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_WM_DESKTOP]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_desktop event", c->name, c->window);
             if (!is_transient)
             {
                 if (ev->data.l[0] == ALL_WORKSPACES)
                 {
-                    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK)
-                        && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+                    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
                     {
                         clientStick (c, TRUE);
                         frameDraw (c, FALSE, FALSE);
@@ -1797,8 +1792,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 }
                 else
                 {
-                    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK)
-                        && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+                    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
                     {
                         clientUnstick (c, TRUE);
                         frameDraw (c, FALSE, FALSE);
@@ -1810,40 +1804,42 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 }
             }
         }
-        else if ((ev->message_type == net_close_window) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_CLOSE_WINDOW]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_close_window event", c->name, c->window);
             clientClose (c);
         }
-        else if ((ev->message_type == net_wm_state) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_WM_STATE]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_state event", c->name, c->window);
             clientUpdateNetState (c, ev);
         }
-        else if ((ev->message_type == net_wm_moveresize) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_WM_MOVERESIZE]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_moveresize event", c->name, c->window);
-            g_message (_("%s: Operation not supported (yet)\n"), g_get_prgname ());
+            g_warning ("Operation not supported (yet)");
             /* TBD */
         }
-        else if ((ev->message_type == net_active_window) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_ACTIVE_WINDOW]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_active_window event", c->name, c->window);
             clientSetWorkspace (c, screen_info->current_ws, TRUE);
             clientShow (c, TRUE);
             clientRaise (c);
-            clientSetFocus (screen_info, c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
-            clientPassGrabMouseButton (c);
+            if (ev->data.l[0] != 0)
+            {
+                clientSetFocus (screen_info, c, (Time) ev->data.l[1], NO_FOCUS_FLAG);
+            }
+            else
+            {
+                clientSetFocus (screen_info, c, CurrentTime, NO_FOCUS_FLAG);
+            }
         }
-        else if (ev->message_type == net_request_frame_extents)
+        else if (ev->message_type == display_info->atoms[NET_REQUEST_FRAME_EXTENTS])
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_request_frame_extents event", c->name, c->window);
-            setNetFrameExtents (myScreenGetXDisplay (screen_info), 
-                                c->window, 
-                                frameTop (c),
-                                frameLeft (c),
-                                frameRight (c),
-                                frameBottom (c)); 
+            setNetFrameExtents (display_info, c->window, frameTop (c), frameLeft (c),
+                                                         frameRight (c), frameBottom (c)); 
         }
     }
     else
@@ -1854,7 +1850,8 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
             return;
         }
         
-        if (((ev->message_type == win_workspace) || (ev->message_type == net_current_desktop)) && (ev->format == 32))
+        if (((ev->message_type == display_info->atoms[WIN_WORKSPACE]) || 
+             (ev->message_type == display_info->atoms[NET_CURRENT_DESKTOP])) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace or a net_current_desktop event");
             if (ev->data.l[0] != screen_info->current_ws)
@@ -1862,33 +1859,32 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 workspaceSwitch (screen_info, ev->data.l[0], NULL);
             }
         }
-        else if (((ev->message_type == win_workspace_count) || (ev->message_type == net_number_of_desktops)) && (ev->format == 32))
+        else if (((ev->message_type == display_info->atoms[WIN_WORKSPACE_COUNT]) || 
+                  (ev->message_type == display_info->atoms[NET_NUMBER_OF_DESKTOPS])) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace_count event");
             if (ev->data.l[0] != screen_info->workspace_count)
             {
                 workspaceSetCount (screen_info, ev->data.l[0]);
-                getDesktopLayout(display_info->dpy, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
+                getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
             }
         }
-        else if ((ev->message_type == net_system_tray_manager) && (ev->data.l[1] == screen_info->net_system_tray_selection)  && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_SYSTEM_TRAY_MANAGER]) && (ev->data.l[1] == screen_info->net_system_tray_selection) && (ev->format == 32))
         {
             TRACE ("root has received a net_system_tray_manager event");
-            screen_info->systray = getSystrayWindow (display_info->dpy,
-                                            screen_info->net_system_tray_selection);
+            screen_info->systray = getSystrayWindow (display_info, screen_info->net_system_tray_selection);
         }
-        else if ((ev->message_type == net_showing_desktop) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_SHOWING_DESKTOP]) && (ev->format == 32))
         {
             TRACE ("root has received a net_showing_desktop event");
             clientToggleShowDesktop (screen_info, ev->data.l[0]);
-            setHint (display_info->dpy, screen_info->xroot, net_showing_desktop, ev->data.l[0]);
+            setHint (display_info, screen_info->xroot, NET_SHOWING_DESKTOP, ev->data.l[0]);
         }
-        else if (ev->message_type == net_request_frame_extents)
+        else if (ev->message_type == display_info->atoms[NET_REQUEST_FRAME_EXTENTS])
         {
             TRACE ("window (0x%lx) has received a net_request_frame_extents event", c->name, ev->window);
             /* Size estimate from the decoration extents */
-            setNetFrameExtents (myScreenGetXDisplay (screen_info), 
-                                ev->window, 
+            setNetFrameExtents (display_info, ev->window, 
                                 frameDecorationTop (screen_info),
                                 frameDecorationLeft (screen_info),
                                 frameDecorationRight (screen_info),
@@ -1941,6 +1937,9 @@ handleEvent (DisplayInfo *display_info, XEvent * ev)
 {
     TRACE ("entering handleEvent");
 
+    /* Update the display time */
+    myDisplayUpdateCurentTime (display_info, ev);
+
     sn_process_event (ev);
     compositorHandleEvent (display_info, ev);
     switch (ev->type)
