@@ -1,4 +1,4 @@
-/*      $Id: main.c,v 1.120 2004/11/01 15:50:24 olivier Exp $
+/*      $Id: main.c,v 1.125 2004/12/08 21:01:57 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -238,13 +238,20 @@ ensure_basedir_spec (void)
             char c;
             
             while ((c = getc (r)) != EOF)
+            {
                 putc (c, w);
+            }
         }
 
         if (r)
+        {
             fclose (r);
+        }
+
         if (w)
+        {
             fclose (w);
+        }
     }
     
     g_free (old);
@@ -255,8 +262,7 @@ ensure_basedir_spec (void)
 
     if (!xfce_mkdirhier(new, 0700, &error)) 
     {
-        g_warning("Unable to create session dir %s: %s",
-                  new, error->message);
+        g_warning("Unable to create session dir %s: %s", new, error->message);
         g_error_free (error);
         g_free (new);
         return;
@@ -416,21 +422,20 @@ initialize (int argc, char **argv, gint compositor_mode)
 
         sn_init_display (screen_info);
         myDisplayAddScreen (display_info, screen_info);
-        setGnomeProtocols (display_info->dpy, screen_info->screen, screen_info->gnome_win);
-        setHint (display_info->dpy, screen_info->xroot, win_supporting_wm_check, screen_info->gnome_win);
-        setHint (display_info->dpy, screen_info->xroot, win_desktop_button_proxy, screen_info->gnome_win);
-        setHint (display_info->dpy, screen_info->gnome_win, win_desktop_button_proxy, screen_info->gnome_win);
-        getHint (display_info->dpy, screen_info->xroot, win_workspace, &ws);
+        setGnomeProtocols (display_info, screen_info->xroot, screen_info->gnome_win);
+        setHint (display_info, screen_info->xroot, WIN_SUPPORTING_WM_CHECK, screen_info->gnome_win);
+        setHint (display_info, screen_info->xroot, WIN_DESKTOP_BUTTON_PROXY, screen_info->gnome_win);
+        setHint (display_info, screen_info->gnome_win, WIN_DESKTOP_BUTTON_PROXY, screen_info->gnome_win);
+        getHint (display_info, screen_info->xroot, WIN_WORKSPACE, &ws);
         screen_info->current_ws = (int) ws;
-        getGnomeDesktopMargins (display_info->dpy, screen_info->screen, screen_info->gnome_margins);
-        set_utf8_string_hint (display_info->dpy, screen_info->gnome_win, net_wm_name, "Xfwm4");
-        setNetSupportedHint (display_info->dpy, screen_info->screen, screen_info->gnome_win);
-        initNetDesktopInfo (display_info->dpy, screen_info->screen, 
-                                   screen_info->current_ws,
+        getGnomeDesktopMargins (display_info, screen_info->xroot, screen_info->gnome_margins);
+        setUTF8StringHint (display_info, screen_info->gnome_win, NET_WM_NAME, "Xfwm4");
+        setNetSupportedHint (display_info, screen_info->xroot, screen_info->gnome_win);
+        initNetDesktopInfo (display_info, screen_info->xroot, screen_info->current_ws,
                                    gdk_screen_get_width (screen_info->gscr), 
                                    gdk_screen_get_height (screen_info->gscr));
         workspaceUpdateArea (screen_info);
-        XSetInputFocus (display_info->dpy, screen_info->gnome_win, RevertToPointerRoot, GDK_CURRENT_TIME);
+        XSetInputFocus (display_info->dpy, screen_info->gnome_win, RevertToPointerRoot, CurrentTime);
 
         clientFrameAll (screen_info);
         
@@ -456,6 +461,7 @@ initialize (int argc, char **argv, gint compositor_mode)
     if (session_init (client_session))
     {
         load_saved_session (client_session);
+        return 1;
     }
 
     return 0;
@@ -490,6 +496,11 @@ main (int argc, char **argv)
     }
 
     status = initialize (argc, argv, compositor);
+    /* 
+       status  < 0   =>   Error, cancel execution
+       status == 0   =>   Run w/out session manager
+       status == 1   =>   Connected to session manager
+     */
     switch (status)
     {
         case -1:
@@ -532,7 +543,8 @@ main (int argc, char **argv)
                 }
 #endif /* !HAVE_DAEMON */
             }
-
+            /* Walk through */
+        case 1:
             /* enter GTK main loop */
             gtk_main ();
             break;
