@@ -1,4 +1,4 @@
-/*      $Id: settings.c,v 1.117 2004/11/06 13:02:11 olivier Exp $
+/*      $Id: settings.c,v 1.121 2004/12/05 23:02:28 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -51,6 +51,10 @@
 
 /* Forward static decls. */
 
+static void              check_for_grabs      (ScreenInfo *);
+static void              set_settings_margin  (ScreenInfo *, 
+                                               int , 
+                                               int);
 static void              notify_cb            (const char *,
                                                const char *,
                                                McsAction, 
@@ -78,6 +82,18 @@ static void              unloadSettings       (ScreenInfo *);
 static gboolean          reloadScreenSettings (ScreenInfo *, 
                                                int);
 
+static void
+check_for_grabs (ScreenInfo *screen_info)
+{
+    if ((screen_info->params->raise_on_click) || (screen_info->params->click_to_focus))
+    {
+        clientGrabMouseButtonForAll (screen_info);
+    }
+    else if (!(screen_info->params->raise_on_click) && !(screen_info->params->click_to_focus))
+    {
+        clientUngrabMouseButtonForAll (screen_info);
+    }
+}
 
 static void
 set_settings_margin (ScreenInfo *screen_info, int idx, int value)
@@ -146,6 +162,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     if (!strcmp (name, "Xfwm/ClickToFocus"))
                     {
                         screen_info->params->click_to_focus = setting->data.v_int;
+                        check_for_grabs (screen_info);
                     }
                     else if (!strcmp (name, "Xfwm/FocusNewWindow"))
                     {
@@ -155,6 +172,18 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         screen_info->params->raise_on_focus = setting->data.v_int;
                     }
+                    else if (!strcmp (name, "Xfwm/MoveOpacity"))
+                    {
+                        screen_info->params->move_opacity = setting->data.v_int;
+                    }
+                    else if (!strcmp (name, "Xfwm/ResizeOpacity"))
+                    {
+                        screen_info->params->resize_opacity = setting->data.v_int;
+                    }
+                    else if (!strcmp (name, "Xfwm/SnapToBorder"))
+                    {
+                        screen_info->params->snap_to_border = setting->data.v_int;
+                    }
                     else if (!strcmp (name, "Xfwm/PreventFocusStealing"))
                     {
                         screen_info->params->prevent_focus_stealing = setting->data.v_int;
@@ -166,12 +195,12 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     else if (!strcmp (name, "Xfwm/RaiseOnClick"))
                     {
                         screen_info->params->raise_on_click = setting->data.v_int;
-                        clientPassGrabMouseButton (NULL);
+                        check_for_grabs (screen_info);
                     }
                     else if (!strcmp (name, "Xfwm/RaiseWithAnyButton"))
                     {
                         screen_info->params->raise_with_any_button = setting->data.v_int;
-                        clientPassGrabMouseButton (NULL);
+                        check_for_grabs (screen_info);
                     }
                     else if (!strcmp (name, "Xfwm/SnapToBorder"))
                     {
@@ -332,9 +361,13 @@ client_event_filter (GdkXEvent * xevent, GdkEvent * event, gpointer data)
     ScreenInfo *screen_info = (ScreenInfo *) data;
     
     if (mcs_client_process_event (screen_info->mcs_client, (XEvent *) xevent))
+    {
         return GDK_FILTER_REMOVE;
+    }
     else
+    {
         return GDK_FILTER_CONTINUE;
+    }
 }
 
 static void
@@ -410,6 +443,7 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
                 &setting) == MCS_SUCCESS)
         {
             setBooleanValueFromInt ("click_to_focus", setting->data.v_int, rc);
+            check_for_grabs (screen_info);
             mcs_setting_free (setting);
         }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/FocusNewWindow", CHANNEL1,
@@ -424,6 +458,18 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
             setBooleanValueFromInt ("raise_on_focus", setting->data.v_int, rc);
             mcs_setting_free (setting);
         }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/MoveOpacity", CHANNEL1,
+                &setting) == MCS_SUCCESS)
+        {
+            setIntValueFromInt ("move_opacity", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/ResizeOpacity", CHANNEL1,
+                &setting) == MCS_SUCCESS)
+        {
+            setIntValueFromInt ("resize_opacity", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/PreventFocusStealing", CHANNEL1,
                 &setting) == MCS_SUCCESS)
         {
@@ -440,12 +486,14 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
                 &setting) == MCS_SUCCESS)
         {
             setBooleanValueFromInt ("raise_on_click", setting->data.v_int, rc);
+            check_for_grabs (screen_info);
             mcs_setting_free (setting);
         }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/RaiseWithAnyButton", CHANNEL1,
                 &setting) == MCS_SUCCESS)
         {
             setBooleanValueFromInt ("raise_with_any_button", setting->data.v_int, rc);
+            check_for_grabs (screen_info);
             mcs_setting_free (setting);
         }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/SnapToBorder", CHANNEL1,
@@ -1068,6 +1116,8 @@ loadSettings (ScreenInfo *screen_info)
         {"margin_right", NULL, FALSE},
         {"margin_bottom", NULL, FALSE},
         {"margin_top", NULL, FALSE},
+        {"move_opacity", NULL, TRUE},
+        {"resize_opacity", NULL, TRUE},
         {"prevent_focus_stealing", NULL, TRUE},
         {"raise_delay", NULL, TRUE},
         {"raise_on_click", NULL, TRUE},
@@ -1208,6 +1258,10 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("raise_on_click", rc));
     screen_info->params->raise_with_any_button =
         !g_ascii_strcasecmp ("true", getValue ("raise_with_any_button", rc));
+    screen_info->params->move_opacity = 
+        abs (TOINT (getValue ("move_opacity", rc)));
+    screen_info->params->resize_opacity = 
+        abs (TOINT (getValue ("resize_opacity", rc)));
 
     screen_info->params->snap_to_border =
         !g_ascii_strcasecmp ("true", getValue ("snap_to_border", rc));
@@ -1368,6 +1422,7 @@ reloadSettings (DisplayInfo *display_info, int mask)
 gboolean
 initSettings (ScreenInfo *screen_info)
 {
+    DisplayInfo *display_info = NULL;
     int i;
     long val = 0;
     char *names = NULL;
@@ -1376,6 +1431,7 @@ initSettings (ScreenInfo *screen_info)
     
     TRACE ("entering initSettings");
 
+    display_info = screen_info->display_info;
     for (i = 0; i < NB_KEY_SHORTCUTS; i++)
     {
         screen_info->params->shortcut_exec[i] = NULL;
@@ -1406,16 +1462,16 @@ initSettings (ScreenInfo *screen_info)
     {
         return FALSE;
     }
-    if (getHint (myScreenGetXDisplay (screen_info), screen_info->xroot, net_number_of_desktops, &val))
+    if (getHint (display_info, screen_info->xroot, NET_NUMBER_OF_DESKTOPS, &val))
     {
         workspaceSetCount (screen_info, val);
     }
-    else if (getHint (myScreenGetXDisplay (screen_info), screen_info->xroot, win_workspace_count, &val))
+    else if (getHint (display_info, screen_info->xroot, WIN_WORKSPACE_COUNT, &val))
     {
         workspaceSetCount (screen_info, val);
     }
 
-    if (getUTF8String (myScreenGetXDisplay (screen_info), screen_info->xroot, net_desktop_names, &names, &i))
+    if (getUTF8String (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &i))
     {
         workspaceSetNames (screen_info, names, i);
     }
@@ -1425,7 +1481,7 @@ initSettings (ScreenInfo *screen_info)
         screen_info->workspace_names_length = 0;
     }
 
-    getDesktopLayout(myScreenGetXDisplay (screen_info), screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
+    getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
     placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
 
     return TRUE;
